package de.stefandraegersoftware.sonar;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;

import org.json.simple.JSONObject;

import de.stefandraegersoftware.wsserver.sonar.SonarService;
import de.stefandraegersoftware.wsserver.sonar.SonarServiceService;
import de.stefandraegersoftware.wsserver.sonar.SonarValue;

@ManagedBean
@SessionScoped
public class ValuePanelController implements Serializable {

	private static final long serialVersionUID = -564815096131096091L;

	private boolean pollValues = false;

	private SonarServiceService sonar;
	private SonarService sonarService;

	private List<SonarValue> sonarValues = new ArrayList<>();

	private float lastValue;

	private int sonarToleranceValue;
	private int sonarMaxDistanceValue;

	private String sonarValueJson;

	private int counter = 0;
	private static boolean up = true;

	private static String NUMBER_FORMAT = "%06.2f";

	private boolean online;

	@PostConstruct
	private void init() {
		try {
			this.sonar = new SonarServiceService();
			this.sonarService = sonar.getSonarServicePort();
			this.sonarValues = sonarService.getSonarValues();
			this.online = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		} finally {
			if (this.sonarValues.isEmpty() || this.sonarValues.size() < 180) {
				for (int i = 0; i < 180; i++) {
					this.sonarValues.add(i, new SonarValue());
				}
			}
		}
	}

	public String readValues() {
		this.sonarValues = sonarService.getSonarValues();
		return SonarStatics.EMPTY;
	}

	public SonarValue getSonarValueByDegree(int degree) {
		SonarValue sonarValue = null;
		sonarValue = sonarService.getSonarValueByIndex(degree);
		return sonarValue;
	}

	public String decrementSonarToleranceValue() {
		this.sonarToleranceValue--;
		return SonarStatics.EMPTY;
	}

	public String incrementSonarToleranceValue() {
		this.sonarToleranceValue++;
		return SonarStatics.EMPTY;
	}

	public String decrementMaxDistanceValue() {
		this.sonarMaxDistanceValue--;
		return SonarStatics.EMPTY;
	}

	public String incrementMaxDistanceValue() {
		this.sonarMaxDistanceValue++;
		return SonarStatics.EMPTY;
	}

	public String actionPollToggle() {
		boolean value = isPollValues();
		setPollValues(!value);
		return SonarStatics.EMPTY;
	}

	public String actionClearValues() {
		return SonarStatics.EMPTY;
	}

	public String actionUpdateValues() {
		return SonarStatics.EMPTY;
	}

	public String actionExportValues() {
		return SonarStatics.EMPTY;
	}

	public void read() {
		if (isOnline()) {
			try {
				SonarValue lastValue = sonarService.getLastSonarValue();
				this.lastValue = lastValue.getValue();
				if (lastValue != null && lastValue.getValue() > this.sonarToleranceValue) {
					this.counter = lastValue.getDegree();
					this.sonarValues.remove(counter);
					this.sonarValues.add(counter, lastValue);
				}
				generateJson();
			} catch (Exception ex) {
				this.online = false;
				ex.printStackTrace();
			}
		}
	}

	public void readSonarGraph() {
		generateJson();
	}

	private void generateJson() {
		JSONObject obj = new JSONObject();
		for (SonarValue value : this.sonarValues) {
			if (value.getValue() > this.sonarToleranceValue) {
				obj.put(value.getDegree(), value.getValue());
			}
		}
		this.sonarValueJson = obj.toJSONString();
	}

	public boolean isPollValues() {
		return pollValues;
	}

	public void setPollValues(boolean pollValues) {
		this.pollValues = pollValues;
	}

	public int getCounter() {
		return counter;
	}

	public void setCounter(int counter) {
		this.counter = counter;
	}

	public List<SonarValue> getSonarValues() {
		return sonarValues;
	}

	public void setSonarValues(List<SonarValue> sonarValues) {
		this.sonarValues = sonarValues;
	}

	public int getSonarToleranceValue() {
		return sonarToleranceValue;
	}

	public void setSonarToleranceValue(int sonarToleranceValue) {
		this.sonarToleranceValue = sonarToleranceValue;
	}

	public boolean isOnline() {
		return online;
	}

	public void setOnline(boolean online) {
		this.online = online;
	}

	public float getLastValue() {
		return lastValue;
	}

	public void setLastValue(float lastValue) {
		this.lastValue = lastValue;
	}

	public int getSonarMaxDistanceValue() {
		return sonarMaxDistanceValue;
	}

	public void setSonarMaxDistanceValue(int sonarMaxDistanceValue) {
		this.sonarMaxDistanceValue = sonarMaxDistanceValue;
	}

	public String getSonarValueJson() {
		return sonarValueJson;
	}

	public void setSonarValueJson(String sonarValueJson) {
		this.sonarValueJson = sonarValueJson;
	}

}
