package de.stefandraegersoftware.wsServer.arduino;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Enumeration;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.stefandraegersoftware.wsServer.sonar.SonarService;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

public class SerialCommunication {
	SerialPort serialPort;

	private static final Logger logger = LogManager.getLogger(SerialCommunication.class);

	private static final String[] AVAILABLE_PORTNAMES = { "COM7" };

	/** Port (Windows) **/
	private static final String PORT = "COM7";

	/**
	 * A BufferedReader which will be fed by a InputStreamReader converting the
	 * bytes into characters making the displayed results codepage independent
	 */
	private BufferedReader input;
	/** Milliseconds to block while waiting for port open */
	private static final int TIME_OUT = 2000;
	/** Default bits per second for COM port. */
	private static final int DATA_RATE = 9600;
	/** **/
	private static final int DATA_BITS = SerialPort.DATABITS_8;
	/** **/
	private static final int STOP_BITS = SerialPort.STOPBITS_1;
	/** **/
	private static final int PARITY = SerialPort.PARITY_NONE;

	/** Instanz des Sonar Service Objektes **/
	private SonarService sonarService;

	/**
	 * Konstruktor
	 * 
	 * @param sonarService
	 */
	public SerialCommunication(SonarService sonarService) {
		this.sonarService = sonarService;
	}

	public void initialize() {
		System.setProperty("gnu.io.rxtx.SerialPorts", PORT);

		CommPortIdentifier portId = null;
		@SuppressWarnings("unchecked")
		Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();

		while (portEnum.hasMoreElements()) {
			CommPortIdentifier currPortId = portEnum.nextElement();
			for (String portName : AVAILABLE_PORTNAMES) {
				if (currPortId.getName().equals(portName)) {
					portId = currPortId;
					break;
				}
			}
		}
		if (portId == null) {
			logger.log(Level.ERROR, "COM Port [" + PORT + "] wurde nicht gefunden!");
			return;
		}

		try {
			serialPort = (SerialPort) portId.open(this.getClass().getName(), TIME_OUT);
			serialPort.setSerialPortParams(DATA_RATE, DATA_BITS, STOP_BITS, PARITY);
			input = new BufferedReader(new InputStreamReader(serialPort.getInputStream()));
			serialPort.addEventListener(new SonarSerialEventListener(sonarService, input));
			serialPort.notifyOnDataAvailable(true);
		} catch (Exception e) {
			logger.log(Level.ERROR, e.getMessage());
		}
	}

	/**
	 * This should be called when you stop using the port. This will prevent
	 * port locking on platforms like Linux.
	 */
	public synchronized void close() {
		if (serialPort != null) {
			serialPort.removeEventListener();
			serialPort.close();
		}
	}

}