package de.stefandraegersoftware.wsServer.arduino;

import java.io.BufferedReader;
import java.io.IOException;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.stefandraegersoftware.wsServer.sonar.SonarService;
import de.stefandraegersoftware.wsServer.sonar.SonarValue;
import de.stefandraegersoftware.wsServer.sonar.SonarValueParser;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;

/**
 * Klasse SonarSerialEventListener dient zum lesen der Daten vom Seriellen Port.
 * 
 * @author Stefan Draeger
 */
public class SonarSerialEventListener implements SerialPortEventListener {
	/** Logger **/
	private static final Logger logger = LogManager.getLogger(SonarSerialEventListener.class);

	/**
	 * Klasse {@link de.stefandraegersoftware.wsServer.sonar.SonarService zum
	 * zwischenspeichern der Daten *
	 */
	private SonarService sonarService;

	/**
	 * BufferedReader welcher den Datenstrom vom Seriellen Port bereitstellt.
	 **/
	private BufferedReader bufferedReader;

	/**
	 * Konstruktor
	 * 
	 * @param bufferedReader
	 *            - {@link java.io.BufferedReader}, enthlt den Datenstrom zum
	 *            Serielen Port.
	 */
	public SonarSerialEventListener(SonarService sonarService, BufferedReader bufferedReader) {
		this.sonarService = sonarService;
		this.bufferedReader = bufferedReader;
	}

	/**
	 * {@see SerialPortEventListener#serialEvent(SerialPortEvent)}
	 */
	public void serialEvent(SerialPortEvent event) {
		// Wenn Daten bereit stehen dann dann diese Auswerten.
		if (event.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
			try {
				// Lesen einer Zeile aus dem Datenstrom.
				String value = bufferedReader.readLine();
				// Wenn der Wert nicht leer ist dann, weiter machen
				if (value.trim().length() > 0) {
					// Den gelesenen Wert parsen und ein SonarValue Objekt
					// daraus generieren.
					SonarValue sonarValue = SonarValueParser.parseFromArduino(value);
					if (sonarValue instanceof SonarValue) {
						logger.log(Level.INFO, sonarValue.toString());
						// System.out.println(sonarValue.toString());
					}
					// Speichern des gelesenen Objektes.
					this.sonarService.addSonarValue(sonarValue);
				}
			} catch (IOException e) {
				// Exception wird nicht gefangen da dieses gelegentlich beim
				// lesen auftritt.
				// logger.log(Level.ERROR, e.getMessage());
			}
		}
	}
}
