package de.stefandraegersoftware.wsServer.sonar;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import javax.jws.WebService;
import javax.jws.WebMethod;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

@WebService
public class SonarService {

	private List<SonarValue> sonarValues = new ArrayList<SonarValue>();

	private SonarValue lastSonarValue = null;
	
	private SonarReader reader;

	private static final Logger logger = LogManager.getLogger(SonarService.class);

	public SonarService() {
		reader = new SonarReader(this);
		init();
		startThread();
	}

	private void init() {
		for (int i = 0; i < 180; i++) {
			this.sonarValues.add(new SonarValue(0, 0, 0L));
		}
	}

	public List<SonarValue> getSonarValues() {
		return sonarValues;
	}

	private void startThread() {
		new Thread(reader).start();
	}

	public SonarReader getReader() {
		return reader;
	}

	public void addSonarValue(SonarValue sonarValue) {
		int degree = sonarValue.getDegree();
		sonarValues.remove(degree);
		sonarValues.add(degree, sonarValue);
		this.lastSonarValue = sonarValue;
	}
	
	@WebMethod
	public SonarValue getLastSonarValue() {
		return lastSonarValue;
	}

	@WebMethod
	public SonarValue getSonarValueByIndex(int index) {
		SonarValue value = null;
		try {
			value = sonarValues.get(index);
		} catch (ArrayIndexOutOfBoundsException ex) {
			logger.log(Level.ERROR, ex);
		}

		return value;
	}

}
