package de.stefandraegersoftware.wsServer.sonar;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public final class SonarValueParser {

	private static final String VALUE_SEPERATOR = ";";

	private static String actualValue;

	private static final Logger logger = LogManager.getLogger(SonarValueParser.class);

	public static SonarValue parseFromArduino(String ardValue) {
		SonarValue sonarValue = null;

		if (ardValue.equalsIgnoreCase("-undefined-") || ardValue.equalsIgnoreCase("")
				|| (!ardValue.startsWith("[") || !ardValue.endsWith("]"))) {
			return null;
		}

		actualValue = ardValue.replace("[", "").replace("]", "");

		try {
			String[] values = actualValue.split(VALUE_SEPERATOR);
			int degree = parseStringToInt(values[0]);
			float value = parseStringToFloat(values[1]);

			if (degree > -1 && value > -1) {
				long timestamp = System.currentTimeMillis();
				sonarValue = new SonarValue(degree, value, timestamp);
			} else {
				sonarValue = null;
			}
		} catch (Exception ex) {
			logger.log(Level.ERROR, ex);
		}
		return sonarValue;
	}

	private static int parseStringToInt(String value) {
		int result = -1;
		if (value.trim().length() > 0) {
			try {
				String tempValue = value.replaceAll("\r\n", "").replaceAll("\n", "");
				result = Integer.parseInt(tempValue);
			} catch (NumberFormatException ex) {
				logger.log(Level.ERROR, ex);
			}
		}

		return result;
	}

	private static float parseStringToFloat(String value) {
		float result = -1f;
		if (value.trim().length() > 0) {
			try {
				String tempValue = value.replaceAll("\r\n", "").replaceAll(",", ".");
				result = Float.parseFloat(tempValue);
			} catch (NumberFormatException ex) {
				logger.log(Level.ERROR, ex);
			}
		}
		return result;
	}
}
